@@ -1,4 +1,4 @@
-from typing import Any, Dict, List, Tuple, TypeVar, Union
+from typing import Any, Callable, Dict, List, Tuple, TypeVar, Union  # noqa
 from urllib.parse import quote
 
 from werkzeug.datastructures import Headers as WerkzeugHeaders
@@ -9,6 +9,7 @@ from werkzeug.urls import url_decode
 
 import ujson as json
 from apistar.pipelines import ArgName
+from apistar.schema import validate
 
 
 class WSGIEnviron(ImmutableDict):
@@ -41,22 +42,30 @@ class Port(int):
         return cls(environ.get('SERVER_PORT') or 80)
 
 
-class RootPath(str):
+class MountPath(str):
     @classmethod
     def build(cls, environ: WSGIEnviron):
         return cls(quote(environ.get('SCRIPT_NAME', '')))
 
 
-class Path(str):
+class RelativePath(str):
     @classmethod
     def build(cls, environ: WSGIEnviron):
         return cls(quote(environ.get('PATH_INFO', '')))
 
 
+class Path(str):
+    @classmethod
+    def build(cls, environ: WSGIEnviron):
+        path = environ.get('SCRIPT_NAME', '') + environ.get('PATH_INFO', '')
+        return cls(quote(path))
+
+
 class QueryString(str):
     @classmethod
     def build(cls, environ: WSGIEnviron):
-        return cls(environ['QUERY_STRING'])
+        query_string = environ.get('QUERY_STRING', '')
+        return cls(query_string)
 
 
 class URL(str):
@@ -88,10 +97,16 @@ class URL(str):
 class Body(bytes):
     @classmethod
     def build(cls, environ: WSGIEnviron):
-        return environ['wsgi.input'].read()
+        content_length = int(environ.get('CONTENT_LENGTH', 0))
+        return environ['wsgi.input'].read(content_length)
 
 
 class Headers(ImmutableHeadersMixin, WerkzeugHeaders):
+    def __init__(self, *args, **kwargs):
+        if len(args) == 1 and isinstance(args[0], dict):
+            args = [list(args[0].items())]
+        super().__init__(*args, **kwargs)
+
     @classmethod
     def build(cls, environ: WSGIEnviron):
         return cls(EnvironHeaders(environ))
@@ -106,11 +121,12 @@ class Header(str):
 class QueryParams(ImmutableMultiDict):
     @classmethod
     def build(cls, environ: WSGIEnviron):
-        return cls(url_decode(environ['QUERY_STRING']))
+        query_string = environ.get('QUERY_STRING', '')
+        return cls(url_decode(query_string))
 
 
 class QueryParam(str):
-    schema = None
+    schema = None  # type: type
 
     @classmethod
     def build(cls, params: QueryParams, arg_name: ArgName):
@@ -118,7 +134,7 @@ class QueryParam(str):
         if value is None or cls.schema is None:
             return value
         if not isinstance(value, cls.schema):
-            value = cls.schema(value)
+            value = validate(cls.schema, value)
         return value
 
 
@@ -135,9 +151,7 @@ ResponseData = TypeVar('ResponseData')
 class Request(object):
     __slots__ = ('method', 'url', 'headers')
 
-    def __init__(self, method: str, url: str, headers: HeadersType=None):
-        if isinstance(headers, dict):
-            headers = list(headers.items())
+    def __init__(self, method: str, url: str, headers: HeadersType=None) -> None:
         self.method = method
         self.url = url
         self.headers = Headers(headers)
@@ -150,13 +164,39 @@ class Request(object):
 class Response(object):
     __slots__ = ('data', 'content', 'status', 'headers')
 
-    def __init__(self, data: Any, status: int=200, headers: HeadersType=None):
-        if isinstance(headers, dict):
-            headers = list(headers.items())
+    def __init__(self, data: Any, status: int=200, headers: HeadersType=None) -> None:
+        if headers is None:
+            headers_dict = {}  # type: Union[Dict[str, str], Headers]
+            headers_list = []  # type: List[Tuple[str, str]]
+        elif isinstance(headers, dict):
+            headers_dict = headers
+            headers_list = list(headers.items())
+        elif isinstance(headers, list):
+            headers_dict = dict(headers)
+            headers_list = headers
+        else:
+            headers_dict = headers
+            headers_list = headers.to_list()
+
+        if isinstance(data, str):
+            content = data.encode('utf-8')
+            content_type = 'text/html; charset=utf-8'
+        elif isinstance(data, bytes):
+            content = data
+            content_type = 'text/html; charset=utf-8'
+        else:
+            content = json.dumps(data).encode('utf-8')
+            content_type = 'application/json'
+
+        if 'Content-Length' not in headers_dict:
+            headers_list += [('Content-Length', str(len(content)))]
+        if 'Content-Type' not in headers_dict:
+            headers_list += [('Content-Type', content_type)]
+
         self.data = data
-        self.content = json.dumps(data).encode('utf-8')
+        self.content = content
         self.status = status
-        self.headers = Headers(headers)
+        self.headers = Headers(headers_list)
 
     @classmethod
     def build(cls, data: ResponseData):
