@@ -19,7 +19,7 @@ WSGIEnviron = http.WSGIEnviron
 class WSGIResponse(object):
     __slots__ = ('status', 'headers', 'iterator')
 
-    def __init__(self, status: str, headers: List[Tuple[str, str]], iterator: Iterable[bytes]):
+    def __init__(self, status: str, headers: List[Tuple[str, str]], iterator: Iterable[bytes]) -> None:
         self.status = status
         self.headers = headers
         self.iterator = iterator
